#!/usr/bin/env python3
import importlib
import time
from ctypes import Structure, c_int8
from math import sin

import rospy
import crc8
from serial import Serial

COMMAND_MODULE_PATH = "/home/k-lab/catkin_ws/devel/lib/python3/dist-packages/rtc2021/msg/_Command.py"
spec = importlib.util.spec_from_file_location("Command", COMMAND_MODULE_PATH)
foo = importlib.util.module_from_spec(spec)
spec.loader.exec_module(foo)
Command = foo.Command


class TxConfig:
    TX_PATH = '/dev/serial/by-id/usb-FTDI_FT230X_Basic_UART_D308ZXNS-if00-port0'


class CommandPackage(Structure):
    _pack_ = 0
    _fields_ = [
        ("wheel_left", c_int8),
        ("wheel_right", c_int8),
        ("arm_rotate", c_int8),
        ("arm_0", c_int8),
        ("arm_1", c_int8),
        ("arm_fist", c_int8),
        ("led", c_int8),
        ("spike_left", c_int8),
        ("spike_right", c_int8)
    ]


def compute_check_sum(input_bytes: bytes) -> int:
    crc = crc8.crc8()
    crc.update(input_bytes)
    check_sum = int(str(crc.hexdigest()), 16)
    return check_sum


def node_tx():
    while not rospy.is_shutdown():
        try:
            rospy.init_node("node_tx")
            rospy.loginfo("TX started")
            ser = Serial(TxConfig.TX_PATH, 57600)
            command_package = CommandPackage()
            rate = rospy.Rate(50)

            def callback(data):
                command_package.wheel_left = int(data.wheel_left * 127)
                command_package.wheel_right = int(data.wheel_right * 127)
                command_package.arm_rotate = int(data.arm_rotate * 127)
                command_package.arm_0 = int(data.arm_0 * 127)
                command_package.arm_1 = int(data.arm_1 * 127)
                command_package.arm_fist = int(data.arm_fist * 127)
                command_package.led = int(data.led * 255)
                command_package.spike_left = int(data.spike_left * 127)
                command_package.spike_right = int(data.spike_right * 127)
                check_sum = compute_check_sum(b'\x00' + bytes(command_package)).to_bytes(1, 'big')
                f = b'\xaa\xaa\x00' + bytes(command_package) + bytes(check_sum)
                ser.write(f)
                ser.flushOutput()

            rospy.Subscriber("/node_tx/command", Command, callback=callback)
            while not rospy.is_shutdown():
                rate.sleep()

        except Exception as e:
            rospy.logerr(e)


if __name__ == "__main__":
    try:
        node_tx()
    except rospy.ROSInterruptException:
        pass
